﻿using Microsoft.Extensions.Logging;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace AirPlaneWithRadar
{
    public partial class FormMapWithSetPlains : Form
    {
        private readonly Dictionary<string, AbstractMap> _mapsDict = new()
        {
            { "Простая карта", new SimpleMap()},{"Карта с большими коробками", new UserMap_BigBox()},{"Карта со стенами", new UserMap_Colums()}
        };
        private readonly MapsCollection _mapsCollection;
        private readonly ILogger _logger;
        public FormMapWithSetPlains(ILogger<FormMapWithSetPlains> logger)
        {
            InitializeComponent();
            _logger = logger; 
            _mapsCollection = new MapsCollection(pictureBox.Width, pictureBox.Height);
            comboBoxSelectorMap.Items.Clear();
            foreach (var elem in _mapsDict)
            {
                comboBoxSelectorMap.Items.Add(elem.Key);
            }
        }
        private void ReloadMaps()
        {
            int index = listBoxMaps.SelectedIndex;

            listBoxMaps.Items.Clear();
            for (int i = 0; i < _mapsCollection.Keys.Count; i++)
            {
                listBoxMaps.Items.Add(_mapsCollection.Keys[i]);
            }

            if (listBoxMaps.Items.Count > 0 && (index == -1 || index >= listBoxMaps.Items.Count))
            {
                listBoxMaps.SelectedIndex = 0;
            }
            else if (listBoxMaps.Items.Count > 0 && index > -1 && index < listBoxMaps.Items.Count)
            {
                listBoxMaps.SelectedIndex = index;
            }
        }

        private void ButtonAddMap_Click(object sender, EventArgs e)
        {
            if (comboBoxSelectorMap.SelectedIndex == -1 || string.IsNullOrEmpty(textBoxNewMapName.Text))
            {
                MessageBox.Show("Не все данные заполнены", "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }
            if (!_mapsDict.ContainsKey(comboBoxSelectorMap.Text))
            {
                MessageBox.Show("Нет такой карты", "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }
            _mapsCollection.AddMap(textBoxNewMapName.Text, _mapsDict[comboBoxSelectorMap.Text]);
            ReloadMaps();
            _logger.LogInformation($"Добавлена карта: {textBoxNewMapName.Text}");
        }

        private void ListBoxMaps_SelectedIndexChanged(object sender, EventArgs e)
        {
            pictureBox.Image = _mapsCollection[listBoxMaps.SelectedItem?.ToString() ?? string.Empty].ShowSet();
            _logger.LogInformation($"Переключение на карту {listBoxMaps.SelectedItem?.ToString() ?? string.Empty}");
        }

        private void ButtonDeleteMap_Click(object sender, EventArgs e)
        {
            if (listBoxMaps.SelectedIndex == -1)
            {
                return;
            }

            if (MessageBox.Show($"Удалить карту {listBoxMaps.SelectedItem}?", "Удаление", MessageBoxButtons.YesNo, MessageBoxIcon.Question) == DialogResult.Yes)
            {
                string nameMap = listBoxMaps.SelectedItem?.ToString() ?? string.Empty;
                _mapsCollection.DelMap(listBoxMaps.SelectedItem?.ToString() ?? string.Empty);
                ReloadMaps();
                _logger.LogInformation($"Удалена карта: {nameMap}");
            }
        }

        private void ButtonAddPlain_Click(object sender, EventArgs e)
        {

            var formPlainConfig = new FormPlaneConfig();
            formPlainConfig.AddEvent(ReactEv);
            formPlainConfig.Show();

        }
        public void ReactEv(DrawingPlain dp)
        {
            if (listBoxMaps.SelectedIndex == -1)
            {
                return;
            }
            try
            {
                if ((_mapsCollection[listBoxMaps.SelectedItem?.ToString() ?? string.Empty] + new DrawingObjectPlane(dp)) >= 0)
                {
                    MessageBox.Show("Объект добавлен");
                    pictureBox.Image = _mapsCollection[listBoxMaps.SelectedItem?.ToString() ?? string.Empty].ShowSet();
                    _logger.LogInformation($"На карту {listBoxMaps.SelectedItem?.ToString() ?? string.Empty} добавлен новый самолет");
                }
                else
                {
                    MessageBox.Show("Не удалось добавить объект");
                    _logger.LogInformation($"Добавление самолета на карту {listBoxMaps.SelectedItem?.ToString() ?? string.Empty} прошло неудачно");
                }
            }
            catch (StorageOverFullException ex)
            {
                MessageBox.Show($"Ошибка добавления : {ex.Message}");
                _logger.LogWarning($"Ошибка добавления самолета на карту {listBoxMaps.SelectedItem?.ToString() ?? string.Empty} : {ex.Message}");
            }
            catch (Exception ex)
            {
                MessageBox.Show($"Неизвестная ошибка : {ex.Message}");
                _logger.LogWarning($"Неизвестная ошибка при добавлении самолета на карту {listBoxMaps.SelectedItem?.ToString() ?? string.Empty} : {ex.Message}");
            }



        }

        private void ButtonRemovePlain_Click(object sender, EventArgs e)
        {
            if (listBoxMaps.SelectedIndex == -1)
            {
                return;
            }
            if (MessageBox.Show("Удалить объект?", "Удаление", MessageBoxButtons.YesNo, MessageBoxIcon.Question) == DialogResult.No)
            {
                return;
            }
            if (maskedTextBoxPosition.Text == null || (maskedTextBoxPosition.Text).Equals(""))
            {
                MessageBox.Show("Упс, что-то пошло не так");
                return;
            }
            int pos = Convert.ToInt32(maskedTextBoxPosition.Text);
            try
            {
                if ((_mapsCollection[listBoxMaps.SelectedItem?.ToString() ?? string.Empty] - (pos - 1)) != null)
                {
                    MessageBox.Show("Объект удален");
                    pictureBox.Image = _mapsCollection[listBoxMaps.SelectedItem?.ToString() ?? string.Empty].ShowSet();
                    _logger.LogInformation($"С карты {listBoxMaps.SelectedItem?.ToString() ?? string.Empty} был удален самолет");

                }
                else
                {
                    MessageBox.Show("Не удалось удалить объект");
                    _logger.LogInformation($"Удаление на карте{listBoxMaps.SelectedItem?.ToString() ?? string.Empty} прошло неудачно");
                }
            }
            catch (PlainNotFoundException ex)
            {
                MessageBox.Show($"Ошибка удаления : {ex.Message}");
                _logger.LogWarning($"Ошибка удаления самолета на карту {listBoxMaps.SelectedItem?.ToString() ?? string.Empty} : {ex.Message}");
            }
            catch (Exception ex)
            {
                MessageBox.Show($"Неизвестная ошибка : {ex.Message}");
                _logger.LogWarning($"Неизвестная ошибка при удалении самолета на карту {listBoxMaps.SelectedItem?.ToString() ?? string.Empty} : {ex.Message}");
            }


        }
        private void ButtonShowStorage_Click(object sender, EventArgs e)
        {
            if (listBoxMaps.SelectedIndex == -1)
            {
                return;
            }
            pictureBox.Image = _mapsCollection[listBoxMaps.SelectedItem?.ToString() ?? string.Empty].ShowSet();
            _logger.LogInformation($"Переход в гараж карты {listBoxMaps.SelectedItem?.ToString() ?? string.Empty}");
        }
        private void ButtonShowOnMap_Click(object sender, EventArgs e)
        {
            if (listBoxMaps.SelectedIndex == -1)
            {
                return;
            }
            pictureBox.Image = _mapsCollection[listBoxMaps.SelectedItem?.ToString() ?? string.Empty].ShowOnMap();
            _logger.LogInformation($"Переход на поле карты {listBoxMaps.SelectedItem?.ToString() ?? string.Empty}");
        }
        private void ButtonMove_Click(object sender, EventArgs e)
        {
            if (listBoxMaps.SelectedIndex == -1)
            {
                return;
            }

            string name = ((Button)sender)?.Name ?? string.Empty;
            Direction dir = Direction.None;
            switch (name)
            {
                case "buttonUp":
                    dir = Direction.Up;
                    break;
                case "buttonDown":
                    dir = Direction.Down;
                    break;
                case "buttonLeft":
                    dir = Direction.Left;
                    break;
                case "buttonRight":
                    dir = Direction.Right;
                    break;
            }
            pictureBox.Image = _mapsCollection[listBoxMaps.SelectedItem?.ToString() ?? string.Empty].MoveObject(dir);
        }

        private void SaveToolStrip_Click(object sender, EventArgs e)
        {
            if (saveFileDialog.ShowDialog() == DialogResult.OK)
            {
                try
                {
                    _mapsCollection.SaveData(saveFileDialog.FileName);
                    MessageBox.Show("Сохранение прошло успешно", "Результат", MessageBoxButtons.OK, MessageBoxIcon.Information);
                    _logger.LogInformation($"Было проведено сохранение в файл : {saveFileDialog.FileName}");

                }
                catch (Exception ex)
                {
                    MessageBox.Show($"Не сохранилось {ex.Message} ", "Результат", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    _logger.LogWarning($"Ошибка при сохранении : {ex.Message}");
                }
            }
        }

        private void loadToolStrip_Click(object sender, EventArgs e)
        {
            if (openFileDialog.ShowDialog() == DialogResult.OK)
            {
                try
                {
                    _mapsCollection.LoadData(openFileDialog.FileName);
                    MessageBox.Show("Загрузка прошла успешно", "Результат", MessageBoxButtons.OK, MessageBoxIcon.Information);
                    ReloadMaps();
                    pictureBox.Image = _mapsCollection[listBoxMaps.SelectedItem?.ToString() ?? string.Empty].ShowSet();
                    _logger.LogInformation($"Была проведена загрузка из файла {openFileDialog.FileName}");
                }
                catch (FileNotFoundException ex)
                {
                    MessageBox.Show($"Не удалось загрузить {ex.Message} ", "Результат", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    _logger.LogWarning($"Ошибка при загрузке из файла {openFileDialog.FileName} : {ex.Message}");
                }
                catch (NullReferenceException ex)
                {
                    MessageBox.Show($"Не удалось загрузить {ex.Message} ", "Результат", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    _logger.LogWarning($"Ошибка при загрузке из файла {openFileDialog.FileName} : {ex.Message}");
                }
                catch (FormatException ex)
                {
                    MessageBox.Show($"Не удалось загрузить {ex.Message} ", "Результат", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    _logger.LogWarning($"Ошибка при загрузке из файла {openFileDialog.FileName} : {ex.Message}");
                }
                catch (Exception ex)
                {
                    MessageBox.Show($"Не удалось загрузить {ex.Message} ", "Результат", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    _logger.LogWarning($"Неизвестная ошибка при загрузке из файла {openFileDialog.FileName} : {ex.Message}");
                }

            }
        }
        private void ButtonSortByType_Click(object sender, EventArgs e)
        {
            if (listBoxMaps.SelectedIndex == -1)
            {
                return;
            }
            _mapsCollection[listBoxMaps.SelectedItem?.ToString() ?? string.Empty].Sort(new PlaneCompareByType());
            pictureBox.Image = _mapsCollection[listBoxMaps.SelectedItem?.ToString() ?? string.Empty].ShowSet();
        }

        private void ButtonSortByColor_Click(object sender, EventArgs e)
        {
            if (listBoxMaps.SelectedIndex == -1)
            {
                return;
            }
            _mapsCollection[listBoxMaps.SelectedItem?.ToString() ?? string.Empty].Sort(new PlaneCompareByColor());
            pictureBox.Image = _mapsCollection[listBoxMaps.SelectedItem?.ToString() ?? string.Empty].ShowSet();
        }

    }
}
