﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace AirPlaneWithRadar
{
    internal class MapsCollection
    {
        readonly Dictionary<string, MapWithSetPlainGeneric<IDrawingObject, AbstractMap>> _mapStorages;

        public List<string> Keys => _mapStorages.Keys.ToList();

        private readonly int _pictureWidth;

        private readonly int _pictureHeight;

        private readonly char separatorDict = '|';

        private readonly char separatorData = ';';

        public int Count { get { return _mapStorages.Count; } }
        public MapsCollection(int pictureWidth, int pictureHeight)
        {
            _mapStorages = new Dictionary<string, MapWithSetPlainGeneric<IDrawingObject, AbstractMap>>();
            _pictureWidth = pictureWidth;
            _pictureHeight = pictureHeight;
        }

        public void AddMap(string name, AbstractMap map)
        {
            
           if (_mapStorages.ContainsKey(name))
                return;
            _mapStorages.Add(name, new MapWithSetPlainGeneric<IDrawingObject, AbstractMap>(_pictureWidth, _pictureHeight, map));

        }

        public void DelMap(string name)
        {
         
            _mapStorages.Remove(name);
        }

        public MapWithSetPlainGeneric<IDrawingObject, AbstractMap> this[string ind]
        {
            get
            {

                if (!_mapStorages.ContainsKey(ind))
                    return null;
                else
                    return _mapStorages[ind];
            }
        }
        public void SaveData(string filename)
        {
            if (File.Exists(filename))
            {
                File.Delete(filename);
            }
            using (StreamWriter sw = new(filename, false))
            {
                sw.WriteLine("MapsCollection");
                foreach (var storage in _mapStorages)
                {
                    sw.WriteLine($"{storage.Key}{separatorDict}{storage.Value.GetData(separatorDict, separatorData)}");
                }
            }

       
        }



        public void LoadData(string filename)
        {
            if (!File.Exists(filename))
            {
                throw new FileNotFoundException("Файл не найден");
            }
            using (StreamReader sr = new StreamReader(filename))
            {
                if (!sr.ReadLine().Equals("MapsCollection"))
                {
                    throw new FormatException("Неверный формат данных в файле");
                }
                _mapStorages.Clear();
                string line = sr.ReadLine();
                while (line != null)
                {
                    var elem = line.Split(separatorDict);
                    AbstractMap map = null;
                    switch (elem[1])
                    {
                        case "SimpleMap":
                            map = new SimpleMap();
                            break;
                        case "UserMap_BigBox":
                            map = new UserMap_BigBox();
                            break;
                        case "UserMap_Colums":
                            map = new UserMap_Colums();
                            break;
                    }
                    _mapStorages.Add(elem[0], new MapWithSetPlainGeneric<IDrawingObject, AbstractMap>(_pictureWidth, _pictureHeight, map));
                    _mapStorages[elem[0]].LoadData(elem[2].Split(separatorData, StringSplitOptions.RemoveEmptyEntries));
                    line = sr.ReadLine();
                }
            }
            if (_mapStorages.Count == 0)
            {
                throw new NullReferenceException("Пустой Файл");
            }

        }
    }
}
