﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace AirPlaneWithRadar
{
    internal class SetPlaneGeneric<T>
     where T : class, IEquatable<T>

    {
        private readonly List<T> _places;
        public int Count => _places.Count;
        private readonly int _maxCount;
        public SetPlaneGeneric(int count)
        {
            _maxCount = count;
            _places = new List<T>();
        }
        public int Insert(T plain)
        {
            if (_places.Contains(plain)) { return -1; }
            Insert(plain, 0);
            return _places.Count;
        }
        public int Insert(T plain, int position)
        {
           
            if (_places.Count == _maxCount)
            {
                throw new StorageOverFullException(_maxCount);
            }
            if (position < 0 || _places.Count < position||position > _maxCount || _places.Count == _maxCount)
                return -1;
            else if (plain == null)
                return -1;
            _places.Insert(position, plain);
            return position;

        }
        public T Remove(int position)
        {
            if (_places[position] == null || (position < 0 || _places.Count < position || position > _maxCount))
            {
                throw new PlainNotFoundException(position);
            }
            T mid;
            
           mid = _places[position];
           _places.RemoveAt(position);
            
            return mid;
        }
        public T this[int position]
        {
            get
            {
                if (position < 0 || _places.Count < position || position > _maxCount)
                    return null;
                else if (_places[position] == null)
                    return null;
                else
                    return _places[position];
            }
            set
            {
                if (position < 0 || _places.Count < position || position > _maxCount)
                    return;
                else if (_places.Count == _maxCount)
                    return;
                else
                    _places[position] = value;
            }
        }
        public IEnumerable<T> GetPlains()
        {
            foreach (var plain in _places)
            {
                if (plain != null)
                    yield return plain;
                else
                    yield break;
            }
        }
        public void SortSet(IComparer<T> comparer)
        {
            if (comparer == null)
                return;
            _places.Sort(comparer);
        }

    }
}
